#include "stdafx.h"
#include "SkpWriterImporter.h"
#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/sketchup/skpwriter/sapi/iapplication.h"
#include "source/sketchup/skpwriter/sapi/idocument.h"
#include "source/sketchup/skpwriter/sapi/ientitycontainer.h"
#include "source/sketchup/skpwriter/sapi/imodel.h"
#include "source/sketchup/skpwriter/sapi/importerdefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

bool GetImporterInfo(SketchUpImporterInfo *pInfo) {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  wsprintf(pInfo->description, L"Foo Files (*.foo)");
  wsprintf(pInfo->extensions, L"foo");
  wsprintf(pInfo->aboutDescription, L"FOO Importer");
  wsprintf(pInfo->identifier, L"my.foo.importer");
  pInfo->supportsOptions = false;
  pInfo->preserveGeometry = false;
  return true;
}

int ConvertToSkp(const wchar_t *inFile, const wchar_t *outFile, int major, int minor, int build) {
  if (major > 13) {
    return kSketchUpVersionNotSupported;
  }

  try {
    HMODULE hMod = LoadLibrary(_T("SkpWriter.dll"));
    if (hMod != NULL) {
      // Get the SkpApplication and container
      typedef atlast::sketchup::ISketchUpApplicationPtr (*Func)();
      Func GetSkpApp = (Func) GetProcAddress(hMod, "GetSkpApplication");
      if (GetSkpApp != NULL) {
        atlast::sketchup::ISketchUpApplicationPtr application = GetSkpApp();
        atlast::sketchup::ISketchUpDocumentPtr document = application->NewDocument();
        atlast::sketchup::ISketchUpModelPtr model = document->GetModel();
        atlast::sketchup::ISketchUpEntityContainerPtr entity_container =
          model->GetRoot();

        // Import the file
        document->StartTransaction(_T("Import"));

        // Set camera position
        atlast::sketchup::CCameraDefinition camera_def;
        camera_def.Set(
          atlast::geometry::CPoint3d(100.0, 100.0, 100.0),
          atlast::geometry::CPoint3d(0.0, 0.0, 0.0),
          atlast::geometry::CUnitVector3d(0.0, 0.0, 1.0));
        model->SetCamera(camera_def);

        // Add geometry  
        entity_container->CreateLineSegment(
          atlast::geometry::CPoint3d(0.0, 0.0, 0.0),
          atlast::geometry::CPoint3d(10.0, 10.0, 0.0));
        atlast::geometry::CPoint3d face_pts[3];
        face_pts[0] = atlast::geometry::CPoint3d(5.0, 5.0, 5.0);
        face_pts[1] = atlast::geometry::CPoint3d(0.0, 5.0, 5.0);
        face_pts[2] = atlast::geometry::CPoint3d(0.0, 0.0, 5.0);
        entity_container->CreateFace(face_pts, 3);

        document->CommitTransaction();
        // save the file - pay attention to the version
        atlast::sketchup::SketchUpFileVersion version = atlast::sketchup::MostRecent;
        document->SaveFile(outFile, version);
      }
      FreeLibrary(hMod);
      return kImportSuccess;
    }
  }
  catch(...) {
  }
  return kImportFail;
}

void ShowSummary() {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
}

void DoOptions() {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
}

BEGIN_MESSAGE_MAP(CSkpWriterImporterApp, CWinApp)
END_MESSAGE_MAP()

CSkpWriterImporterApp::CSkpWriterImporterApp()
{
}

CSkpWriterImporterApp theApp;

BOOL CSkpWriterImporterApp::InitInstance()
{
	CWinApp::InitInstance();

	return TRUE;
}
