// Copyright 2013 Trimble Navigation Limited. All Rights Reserved.

#include "stdafx.h"
#include <string>
#include "SkpWriterImporterShimPlugin.h"

SkpWriterImporterShimPlugin *SkpWriterImporterShimPlugin::s_pInstance = NULL;

// Constructor loads the dependent DLLs
SkpWriterImporterShimPlugin::SkpWriterImporterShimPlugin() {
  // Xerces is here first so that it will already be in memory when
  // SkpWriter.dll is loaded, which allows it to be in the Importers folder,
  // and doesn't need to be in the same directory as SketchUp.exe
  m_XercesDLL = LoadLibrary(_T("Importers\\xerces-c_2_6.dll"));
  // Load the SkpWriter DLL
  m_SkpWriterDLL = LoadLibrary(_T("Importers\\SkpWriter.dll"));
  // Load the SU8 plugin DLL
  m_ImporterDLL = LoadLibrary(_T("Importers\\SkpWriterImporter.dll"));
  // Load the properties of the legacy extension
  LoadInfo();
}

// Unloads the dependent DLLs so we don't keep them loaded unnecessarily
SkpWriterImporterShimPlugin::~SkpWriterImporterShimPlugin() {
  FreeLibrary(m_XercesDLL);
  FreeLibrary(m_SkpWriterDLL);
  FreeLibrary(m_ImporterDLL);
}

// Invokes the legacy extension GetImporterInfo() function
void SkpWriterImporterShimPlugin::LoadInfo() {
  if ((m_XercesDLL != NULL) && (m_ImporterDLL != NULL))
  {
    typedef bool (*Func)(SketchUpImporterInfo *pInfo);
    Func GetImporterInfo =
        (Func) GetProcAddress(m_ImporterDLL, "GetImporterInfo");
    if (GetImporterInfo != NULL) {
      GetImporterInfo(&m_info);
    }
  }
}

std::string SkpWriterImporterShimPlugin::GetDescription() const {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  USES_CONVERSION;
  // Converting to MBCS for brevity. For unicode descriptions, you must
  // convert to UTF-8.
  return W2A(m_info.description);
}

std::string SkpWriterImporterShimPlugin::GetIdentifier() const {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  USES_CONVERSION;
  char* retval = W2A(m_info.identifier);
  return retval;
}

std::string SkpWriterImporterShimPlugin::GetFileExtension(int index) const {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  USES_CONVERSION;
  return W2A(m_info.extensions);
}

bool SkpWriterImporterShimPlugin::ConvertToSkp(const std::string& inFile,
    const std::string& outFile, SketchUpPluginProgressCallback* callback,
    void* reserved) {
  AFX_MANAGE_STATE(AfxGetStaticModuleState());
  USES_CONVERSION;

  int retVal = kImportFail;
  if ((m_XercesDLL != NULL) && (m_ImporterDLL != NULL))
  {
    typedef int (*Func)(const wchar_t *inFile, const wchar_t *outFile,
        int major, int minor, int build);
    Func ConvertFunction =
        (Func) GetProcAddress(m_ImporterDLL, "ConvertToSkp");
    if (ConvertFunction != NULL) {
      // Call the legacy extension's conversion function, which should produce
      // a temporary .skp file that SketchUp will import into the active model
      retVal = ConvertFunction(A2W(inFile.c_str()),
                               A2W(outFile.c_str()), 8, 0, 0);
    }
  }
  return retVal == kImportSuccess;
}

//////////////////////////////////////////////////////////////////////////
// Static functions used to create and destroy the actual plugin object

SkpWriterImporterShimPlugin *SkpWriterImporterShimPlugin::GetInstance() {
  if (s_pInstance == NULL) {
    s_pInstance = new SkpWriterImporterShimPlugin();
  }
  return s_pInstance;
}

void SkpWriterImporterShimPlugin::DestroyInstance() {
  if (s_pInstance) {
    delete s_pInstance;
    s_pInstance = NULL;
  }
}
