// Copyright 2010 Google Inc. All Rights Reserved.

// Because UNICHAR is not a default type for stl, we need to define char_traits
// for it on the Mac.

#ifndef SKETCHUP_SOURCE_COMMON_UTILS_USTRING_TRAITS_H_
#define SKETCHUP_SOURCE_COMMON_UTILS_USTRING_TRAITS_H_

#if defined(__APPLE__) || defined(__LINUX__)

#include <cstdio>
#include <cstring>
#include "source/common/utils/unichar.h"

namespace std {
    
    template<>
    struct char_traits<UNICHAR>
{
    typedef UNICHAR   	char_type;
    typedef int 	    int_type;
    typedef streampos 	pos_type;
    typedef streamoff 	off_type;
    typedef mbstate_t 	state_type;
    
    static void 
        assign(char_type& __c1, const char_type& __c2)
    { __c1 = __c2; }
    
    static bool 
        eq(const char_type& __c1, const char_type& __c2)
    { return __c1 == __c2; }
    
    static bool 
        lt(const char_type& __c1, const char_type& __c2)
    { return __c1 < __c2; }
    
    static int 
        compare(const char_type* __s1, const char_type* __s2, size_t __n)
    { return ustrncmp(__s1, __s2, __n); }
    
    static size_t
        length(const char_type* __s)
    { return ustrlen(__s); }
    
    static const char_type* 
        find(const char_type* __s, size_t __n, const char_type& __a)
    { return umemchr(__s, __a, __n); }
    
    static char_type* 
        move(char_type* __s1, const char_type* __s2, size_t __n)
    { return static_cast<char_type*>(memmove(__s1, __s2, 2*__n)); }
    
    static char_type* 
        copy(char_type* __s1, const char_type* __s2, size_t __n)
    {  return ustrncpy(__s1, __s2, __n); }
    
    static char_type* 
        assign(char_type* __s, size_t __n, char_type __a)
    { return umemset(__s, __a, __n); }
    
    static char_type 
        to_char_type(const int_type& __c)
    { return static_cast<char_type>(__c); }
    
    // To keep both the byte 0xff and the eof symbol 0xffffffff
    // from ending up as 0xffffffff.
    static int_type 
        to_int_type(const char_type& __c)
    { return static_cast<int_type>(__c); }
    
    static bool 
        eq_int_type(const int_type& __c1, const int_type& __c2)
    { return __c1 == __c2; }
    
    static int_type 
        eof() { return static_cast<int_type>(UEOF); }
    
    static int_type 
        not_eof(const int_type& __c)
    { return (__c == eof()) ? 0 : __c; }
};

}  // namespace std

#endif  // __APPLE__ or __LINUX__
#endif  // SKETCHUP_SOURCE_COMMON_UTILS_USTRING_TRAITS_H_
