// $Header: /home/cvs/refactor/source/sketchup/macwin/mfcbase.h,v 1.8 2007/04/14 19:35:00 mikes Exp $
//=============================================================================
// Copyright 1999-2001 @Last Software
//=============================================================================

// Basic MFC definitions for the MAC

#ifndef _mfcbase_h_
#define _mfcbase_h_

#if defined(__APPLE__) || defined(__LINUX__)
//  mfc.h
//  
//  Created by joe on Fri Aug 10 2001.

#include "source/common/utils/mac/tchar.h"
#include "source/common/utils/mac/wintype.h"
#include "source/sketchup/macwin/comtype.h"
#include "source/sketchup/macwin/windef.h"

//=============================================================================

inline int AfxIsValidString(LPCTSTR s, int l = -1) { return (NULL != s); }
inline int AfxIsValidAddress(const void* p, UINT n, BOOL rw=TRUE) {return (NULL != p);}
inline BOOL AfxEnableMemoryTracking(BOOL flag) { return FALSE; }

#ifdef _DEBUG
#include <assert.h>

#ifndef TRACE
extern void AfxTrace(LPCSTR, ...);
extern void AfxTrace(LPCWSTR, ...);
#define TRACE ::AfxTrace
#define TRACE0(sz) ::AfxTrace(_T("%s"), _T(sz))
#endif

#define THIS_FILE __FILE__
#ifndef ASSERT
#define ASSERT(f) assert(f)
#endif
#define VERIFY(f) ASSERT(f)
#define ASSERT_VALID(pObj) ASSERT(NULL != (pObj))

#else

#ifndef TRACE
inline void AfxTrace(LPCSTR, ...) { }
inline void AfxTrace(LPCWSTR, ...) { }
#define TRACE 1 ? (void)0 : ::AfxTrace
#define TRACE0(sz)
#endif

#ifndef ASSERT
#define ASSERT(f)\
((void) ((f) ? 0 : printf("%s:%d: failed assertion `%s'\n",__FILE__,__LINE__,#f)))
#endif

#define VERIFY(f)          ((void)(f))
#define ASSERT_VALID(pOb)  ((void)0)

#endif // _DEBUG

extern void OutputDebugString(LPCTSTR s);

//-------------------------------------------------------------------

#define _AFX_INLINE inline

//=============================================================================

// POSITION is an obscure type used in the MFC collection objects.
// On the Mac, I have tried to replace the MFC collections with STL collections.
// There are a couple places in ported MFC code however where it is hard to
// replace them, so in those cases I am implementing the MFC collections using
// STL collections.  This stuff is used to go from an STL iterator to an 
// MFC POSITION.

// POSITION is returned by value from MFC collections, so it has to be a known
// size, so it is hard to use polymorphism on it.  For this reason, we use
// POSITION to simple wrap another polymorphic class.  Since it is assumed
// that POSITION is an automatic variable, we use reference counting to
// free the iterator object.

class POSITION;

class CPositionIterator
{
protected:
    CPositionIterator() : m_nCount(0) {}
    virtual ~CPositionIterator() {}
    
    long AddRef() { m_nCount++; return m_nCount;}
    long Release()
    {
        long n = --m_nCount;
        if(m_nCount <= 0) delete this;
        return n;
    }
    
    void AddToPosition(POSITION& pos);
    friend class POSITION;
    
    long m_nCount;
};

class POSITION
{
public:
    POSITION() : m_pIter(NULL) { }
    POSITION(const POSITION& pos) : m_pIter(NULL) { SetIterator(pos.m_pIter); }
    POSITION(CPositionIterator* pIter) : m_pIter(NULL) { SetIterator(pIter); }
    ~POSITION() { if( m_pIter ) m_pIter->Release(); }
    POSITION& operator=(const POSITION& pos)
    { SetIterator(pos.m_pIter); return *this; }

    bool operator==(int null) { ASSERT(null==0); return m_pIter == NULL; }
    bool operator!=(int null) { ASSERT(null==0); return m_pIter != NULL; }
    operator bool() { return m_pIter != NULL; }
    
    CPositionIterator* GetIterator() { return m_pIter; }
    void SetIterator(CPositionIterator* pIter)
    {
        if( m_pIter ) m_pIter->Release();
        m_pIter = pIter;
        if( m_pIter ) m_pIter->AddRef();
    }
    
protected:
    friend class CPositionIterator;
    CPositionIterator* m_pIter;
};

//=============================================================================
#endif // __APPLE__
#endif // _mfcbase_h_

