// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_

#define THROWS(something)

// AtLast
#include "source/common/geometry/geometry.h"
#include "source/common/geometry/boundingbox.h"
#include "source/common/utils/color.h"
// SketchUp
#include "source/sketchup/utils/smartptr.h"
#include "source/sketchup/utils/smartvector.h"
#include "source/sketchup/utils/ivectorofsmartpointers.h"

#include <typeinfo>
#include <vector>
#include <set>
#include <string>



// do this in the sketchup namespace

namespace atlast {
namespace sketchup {
// Now all the smart pointer types
// a utility
#define DECLARE_SMART_POINTER3(_type)                   \
    class _type;                                        \
    typedef atlast::utils::SmartPtr<_type*> _type##Ptr;

#define DECLARE_SMART_POINTER_NO_FORWARD_DECLARE(_type) \
   typedef atlast::utils::SmartPtr< _type* > _type##Ptr;


#define DECLARE_VECTOR_OF(_type)                \
    typedef IVectorOfSmartPointers< _type, _type##Ptr > _type##Vector; \
    typedef SmartVector<_type, _type##Ptr, _type##Vector> _type##VectorImpl;

#define DECLARE_SMART_POINTER_TO_VECTOR_OF(_type) \
    DECLARE_VECTOR_OF(_type) \
    DECLARE_SMART_POINTER_NO_FORWARD_DECLARE(_type##Vector)


#define DECLARE_SMART_POINTER_AND_VECTOR(_type) \
    DECLARE_SMART_POINTER3(_type) \
    DECLARE_SMART_POINTER_TO_VECTOR_OF(_type)


/**
  \namespace atlast::sketchup
  \brief The API classes.
 */


// now all the smart pointer types -
// Review: should these go in their own header files?

/**
 * @class ISketchUpObjectPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpObject class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpObject)

/**
 * @class ISketchUpApplicationPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpApplication class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpApplication)
/**
 * @class ISketchUpModelPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpModel class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpModel)
/**
 * @class ISketchUpGeometricElementPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpGeometricElement class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpGeometricElementVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpGeometricElement objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpGeometricElement)
/**
 * @class ISketchUpEntityPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpEntity class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpEntityVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpEntity objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpEntity)

/**
 * @class ISketchUpEntityContainerPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpEntityContainer class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpEntityContainer)
/**
 * @class ISketchUpVertexPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpVertex class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpVertexVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpVertex objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpVertex)

/**
 * @class ISketchUpEdgePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpEdge class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpEdgeVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpEdge objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpEdge)
/**
 * @class ISketchUpEdgeUsePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpEdgeUse class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpEdgeUseVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpEdgeUse objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpEdgeUse)
/**
 * @class ISketchUpLayerPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpLayer class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpLayer)
/**
 * @class ISketchUpAttributeDictionaryPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpAttributeDictionary class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpAttributeDictionary)
/**
 * @class ISketchUpAttributeDictionaryEntryPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpAttributeDictionaryEntry class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpAttributeDictionaryEntry)
/**
 * @class ISketchUpComponentInstancePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpComponentInstance class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpComponentInstance)
/**
 * @class ISketchUpComponentDefinitionPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpComponentDefinition class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpComponentDefinition)
/**
 * @class ISketchUpFacePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpFace class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpFaceVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpFace objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpFace)
/**
 * @class ISketchUpPolylinePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpPolyline class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpPolyline)
/**
 * @class ISketchUpOptionsProviderPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpOptionsProvider class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpOptionsProvider)
/**
 * @class ISketchUpOptionsManagerPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpOptionsManager class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpOptionsManager)
/**
 * @class ISketchUpTexturePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpTexture class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpTexture)
/**
 * @class ISketchUpMaterialPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpMaterial class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpMaterial)
/**
 * @class ISketchUpArcPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpArc class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpArc)
/**
 * @class ISketchUpLineSegmentPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpLineSegment class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpLineSegmentVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpLineSegment objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpLineSegment)
/**
 * @class ISketchUpLoopPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpLoop class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpLoopVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpLoop objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpLoop)
/**
 * @class ISketchUpToolPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpTool class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpTool)
/**
 * @class ISketchUpViewPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpView class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpView)
/**
 * @class ISketchUpDocumentPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpDocument class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpDocument)
/**
 * @class ISketchUpSelectionSetPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpSelectionSet class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpSelectionSet)
/**
 * @class ISketchUpInputPointPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpInputPoint class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpInputPoint)
/**
 * @class ISketchUpMenuPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpMenu class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpMenu)
/**
 * @class ISketchUpMenuCallbackPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpMenuCallback class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpMenuCallback)
/**
 * @class ISketchUpControllerPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpController class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpController)
/**
 * @class ISketchUpCurvePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpCurve class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpCurveVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpCurve objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpCurve)
/**
 * @class ISketchUpDrawingElementPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpDrawingElement class.
 * @see SmartPtr
 */
/**
 * \class ISketchUpDrawingElementVectorPtr
 * \brief A smart pointer wrapper around a "vector" of 
 * ISketchUpDrawingElement objects.
 * \see IVectorOfSmartPointers
 */
DECLARE_SMART_POINTER_AND_VECTOR(ISketchUpDrawingElement)
/**
 * @class ISketchUpAngleFormatterPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpAngleFormatter class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpAngleFormatter)
/**
 * @class ISketchUpLengthFormatterPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpLengthFormatter class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpLengthFormatter)
/**
 * @class ISketchUpGroupPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpGroup class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpGroup)
/**
 * @class ISketchUpPickHelperPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpPickHelper class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpPickHelper)
/**
 * @class ISketchUpComponentBehaviorPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpComponentBehavior class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpComponentBehavior)

/**
 * @class ISketchUpPagePtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpPage class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpPage)

/**
 * @class ISketchUpPagesPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpPages class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpPages)


/**
 * @class ISketchUpMaterialsPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpMaterials class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpMaterials)
/**
 * @class ISketchUpLayersPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpLayers class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpLayers)
/**
 * @class ISketchUpDefinitionsPtr
 * @brief A smart pointer wrapper around the atlast::sketchup::ISketchUpDefinitions class.
 * @see SmartPtr
 */
DECLARE_SMART_POINTER3(ISketchUpDefinitions)
}  // namespace atlast
}  // namespace sketchup

atlast::sketchup::ISketchUpApplicationPtr GetSkpApplication();
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_
