// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IANGLEFORMATTER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IANGLEFORMATTER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/common/utils/ustring.h"  // AtLast
namespace atlast {
namespace sketchup {
class ISketchUpAngleFormatter;
class ISketchUpInputPoint;

/**
 * @class ISketchUpAngleFormatter
 * @brief A utility class that helps with formatting angles.
 */
class ISketchUpAngleFormatter :
  public ISketchUpObject {
public:
  /**
   * @brief Formats the given value into the current angle
   * format.
   * @param angle The angle in radians normalized between
   * 0 and +/- 2PI.
   * @return The formatted angle.
   */
  virtual AtLastUstring Format(double angle) const = 0;

  /**
   * @brief Parses the given string representation of an angle, putting
   * the result in the given double value.
   * 
   * This method will attempt to parse the given string which should represent
   * an angle.  The result goes in the given double value if not NULL.
   *
   * @param strValue The string representation to be parsed.
   * @param pVal The optional parameter that will be populated with the 
   * parsed value if not NULL and if the strValue parameter is valid.
   * @return true iff the value could be parsed as an angle.
   */
  virtual bool Parse(LPCWSTR strValue, double *pVal) const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IANGLEFORMATTER_H_
