// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IAPPLICATION_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IAPPLICATION_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/sketchup/skpwriter/sapi/iobject.h"

#if (defined __APPLE__) || (defined __LINUX__)
#include "source/sketchup/macwin/CString.h"
#endif

namespace atlast {
namespace sketchup {
/**
 * @interface ISketchUpApplication
 * @brief The Application providing all the underlying objects.
 *
 * This class supports any UI that applies across all underlying
 * documents.
 */
class ISketchUpApplication :
  public ISketchUpObject {
public:
  virtual ~ISketchUpApplication() {}

  /**
   * @brief Retrieves the current document.
   * @retval The current document associated with the Application.
   */
  virtual ISketchUpDocumentPtr GetActiveDocument() = 0;

  /**
   * \brief Creates a new document.  Not implemented in all versions of
   * the application.
   * \return The newly created document.
   */
  virtual ISketchUpDocumentPtr NewDocument() = 0;

  /**
   * @brief Provides access to <i>system</i> menus.
   *
   * This method will return a menu associated with the <i>well known
   * name</i> of the system menus.
   *
   * @param menuName The name of the system menu.
   * @retval The system menu, or NULL if not found.
   */
  virtual ISketchUpMenuPtr GetMenu(LPCWSTR menuName) = 0;

  /**
   * \brief Returns the version of SketchUp.
   *
   * The return value is in the format: major.minor.build
   * \retval The version of this application.
   */
  virtual CString GetVersion() const = 0;
};
}
}
//typedef sketchup::SmartPtr<sketchup::IApplication> sketchup::IApplicationPtr;
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IAPPLICATION_H_
