// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IATTRIBUTEDICTIONARY_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IATTRIBUTEDICTIONARY_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/utils/typedvalue.h"  // SketchUp
#include "source/sketchup/skpwriter/sapi/iattribute.h"
#include "source/sketchup/skpwriter/sapi/iattributedictionaryentry.h"
namespace atlast {
namespace sketchup {
class ISketchUpAttributeDictionaryEntry;

/**
 * @interface ISketchUpAttributeDictionary
 * @brief A collection of name/value pairs.
 *
 * An ISketchUpAttributeDictionary is a special kind of Attribute
 * that can be attached to an Entity.  This is a collection of
 * name value pairs.  It is useful when categorizing attribute
 * types. The <i>values</i> are simple types: one cannot nest
 * AttributeDictionaries.
 */
class ISketchUpAttributeDictionary :
  public ISketchUpAttribute {
public:
  virtual ISketchUpAttributeDictionaryEntryPtr
    CreateAttributeDictionaryEntry(LPCWSTR name,
                                   const atlast::utils::CTypedValue &value) =
    0;

  virtual atlast::utils::CTypedValue GetValue(LPCWSTR name) const = 0;
  virtual ISketchUpAttributeDictionaryEntryPtr GetEntry(LPCWSTR name) const =
    0;

  virtual ISketchUpAttributeDictionaryEntryPtr
    CreateAttributeDictionaryEntry(LPCWSTR name,
                                   ISketchUpAttributeDictionaryEntry::ValueType
                                   type, double x, double y, double z) = 0;

  virtual ISketchUpAttributeDictionaryEntryPtr
    CreateAttributeDictionaryEntry(LPCWSTR name, LPCWSTR value) = 0;

  virtual ISketchUpAttributeDictionaryEntryPtr
    CreateAttributeDictionaryEntry(LPCWSTR name, double value) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IATTRIBUTEDICTIONARY_H_
