// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTBEHAVIOR_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTBEHAVIOR_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"
namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpComponentBehavior
 * @brief Represents a ComponentBehavior in SketchUp.
 */
class ISketchUpComponentBehavior :
  public ISketchUpEntity {
public:
  /**
   * \brief Property as to whether the object is "2D"
   * \retval true iff this is a 2D object.
   */
  virtual bool Is2D() const = 0;
  /**
   * \brief Set the 2D property of this behavior object.
   * \param is2d Whether or not this object should behave as 2D.
   */
  virtual void Set2D(bool is2d) = 0;

  /**
   * \brief Does this object 'cut' an opening?
   * \retval true iff this object cuts.
   */
  virtual bool CutsOpening() const = 0;
  /**
   * \brief Set the 'cuts' property of this behavior object.
   * \param cuts Whether or not this object should cut.
   */
  virtual void SetCutsOpening(bool cuts) = 0;

  typedef enum {
    Arbitrary  = 0,
    Horizontal = 1,
    Vertical   = 2,
    Sloped     = 3
  } SnapType;
  /**
   * \brief Returns the type of face this object will snap to.
   * \retval One of the SnapType enumeration values.
   */
  virtual SnapType GetSnapTo() const = 0;
  /**
   * \brief Set the 'SnapTo' property of this behavior object.
   * \param type The type of snap behavior for this object.
   */
  virtual void SetSnapTo(SnapType type) = 0;

  /**
   * \brief Does this object always face the camera?
   *
   * That is, is this a "faceme" object?
   * \retval true iff this object always faces the camera.
   */
  virtual bool IsFaceMe() const = 0;
  /**
   * \brief Set the 'faceme' property of this behavior object.
   * \param faceme The value for the FaceMe property.
   */
  virtual void SetFaceMe(bool faceme) = 0;

  /**
   * \brief Do this object's shadows always face the sun?
   *
   * \retval true iff this object's shadows always faces the sun.
   */
  virtual bool DoShadowsFaceSun() const = 0;
  /**
   * \brief Set the 'ShadowsFaceSun' property of this behavior object.
   * \param facesun The value for the ShadowsFaceSun property.
   */
  virtual void SetShadowsFaceSun(bool facesun) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTBEHAVIOR_H_
