// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTDEFINITION_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTDEFINITION_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientitycontainer.h"
#include "source/sketchup/skpwriter/sapi/idrawingelement.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/vector3d.h"
namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpComponentDefinition
 * @brief A Component Definition class.
 */
class ISketchUpComponentDefinition :
  public ISketchUpDrawingElement {
public:
  virtual atlast::geometry::CBoundingBox3d GetBoundingBox() const = 0;
  virtual void OrientFacesConsistently() = 0;
  virtual void SetInsertPoint(const atlast::geometry::CPoint3d &pnt) = 0;
  virtual void SetCoordinateAxes(const atlast::geometry::CPoint3d &origin,
                                 const atlast::geometry::CUnitVector3d &xAxis,
                                 const atlast::geometry::CUnitVector3d &yAxis)
    = 0;

  virtual ISketchUpEntityContainerPtr GetEntityContainer() const = 0;

  /**
   * \brief Returns the component behavior object associated with this definition.
   * \retval The ISketchUpComponentBehavior object associated with this definition.
   * Returns NULL is either this definition isn't valid or there is no behavior object so associated.
   */
  virtual ISketchUpComponentBehaviorPtr GetComponentBehavior() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTDEFINITION_H_
