// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTINSTANCE_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTINSTANCE_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/idrawingelement.h"
#include "source/common/utils/ustring.h"  // AtLast
namespace atlast {
namespace sketchup {

/**
 * @class ISketchUpComponentInstance
 * @brief A Component Instance class.
 */

class ISketchUpComponentInstance :
  public ISketchUpDrawingElement {
public:
  /**
   * @brief Explodes this Component Instance into its constituent pieces.
   * @return A vector of the entities that made up the component.
   */
  virtual ISketchUpEntityVectorPtr Explode() = 0;
  /**
   * @brief Retrieves this instance's defintion.
   *
   * @return The Component Definition of which this is an instance.
   */
  virtual ISketchUpComponentDefinitionPtr GetComponentDefinition() const = 0;

  /**
   * @brief Retrieves the transformation associated with this instance.
   *
   * The actual geometry associated with this instance is obtained via the
   * definition (see GetComponentDefinition()) and this transform.
   *
   * To transform this instance, use the ISketchUpController::Transform()
   * method.
   *
   * @return The transformation that takes the definition's geometry to 
   * this instance's geometry.
   */
  virtual atlast::geometry::CTransformation GetTransformation() const = 0;

  /**
   * \brief Sets the transform of the component instance relative its definition.
   *
   * This method should be contrasted with ISketchUpController::Transform(), where
   * the component instance would be transformed from its current location by the
   * given transformation.  In this case, its current location is overwritten, and
   * its location - relative to its definition - is set explicitly.  
   *
   * This method is particularly useful when creating new component instances, such
   * as in the case of writing an Importer.
   *
   * \param trans The transform from the definition to the desired location of the
   * instance.
   */
  virtual void SetTransform(const atlast::geometry::CTransformation &trans) =
    0;

  virtual AtLastUstring GetName() const = 0;
  virtual void SetName(LPCWSTR name) = 0;

  virtual void SetLocked(bool lock) = 0;
  virtual bool IsLocked() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICOMPONENTINSTANCE_H_
