// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDEFINITIONS_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDEFINITIONS_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/common/utils/ustring.h"  // AtLast

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpPage
 * @brief A <i>Page</i> is a named view in SketchUp.
 *
 */
class ISketchUpDefinitions :
  public ISketchUpEntity {
public:
  /**
   * \brief The number of Definitions in this collection
   */
  virtual int GetCount() const = 0;

  /**
   * \brief The Definition at the given index.
   * \param indx The index of the desired Definition.
   * \return The Definition at the given index or NULL if a bad index was provided.
   */
  virtual ISketchUpComponentDefinitionPtr GetAt(int indx) const = 0;

  /**
   * \brief Create a new Definition.
   *
   * The given name may be munged to ensure we create a uniquely named Definition.
   */
  virtual ISketchUpComponentDefinitionPtr CreateComponentDefinition
    (LPCWSTR name = NULL) = 0;

  /**
   * \brief Load a component definition from a file.
   * 
   * \return The loaded Definition, NULL if something goes wrong.
   */
  virtual ISketchUpComponentDefinitionPtr LoadComponentDefinition
    (LPCWSTR fileName) = 0;

  /**
   * \brief Find the component definition with the given name.
   */
  virtual ISketchUpComponentDefinitionPtr FindComponentDefinition
    (LPCWSTR name) = 0;

  /**
   * Purge the list of unused Definitions.
   */
  virtual void Purge() = 0;

  /**
   * Delete the given Definition.
   */
  virtual void DeleteComponentDefinition(ISketchUpComponentDefinition
                                         *pDefinition) = 0;

  /**
   * Generate a unique name.
   */
  virtual AtLastUstring GenerateUniqueComponentDefinitionName(LPCWSTR baseName =
                                                        NULL) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDEFINITIONS_H_
