// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDRAWINGELEMENT_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDRAWINGELEMENT_H_
#pragma once

#include "source/sketchup/skpwriter/sapi/igeometricelement.h"

namespace atlast {
namespace sketchup {

class ISketchUpLayer;
class ISketchUpMaterial;

/**
 * @class ISketchUpDrawingElement
 * @brief An object that appears on the screen.
 */
class ISketchUpDrawingElement :
  public ISketchUpGeometricElement {
public:
  /**
   * @brief Set this object's visibility flag.
   *
   * If set to visible, this object still might not show up
   * in the view - it's also a function of its component hierarchy
   * and layer membership. See \ref VC.
   */
  virtual void SetVisible(bool isVisible) = 0;
  /**
   * @brief Get this object's visibility flag.
   * See \ref VC.
   * @return The visibility flag on this particular object.
   */
  virtual bool IsVisible() const         = 0;

  /**
   * @brief Used to set the material on an object.
   *
   * If the given parameter is NULL, the material on the object is
   * removed and the default material determination is used.
   * See \ref MAT.
   * @param pMaterial The material to use.
   */
  virtual void SetMaterial(const ISketchUpMaterial *pMaterial) THROWS(ISketchUpException&)
    = 0;

  /**
   * \brief Set the layer for the current object.
   * \param pLayer The layer to put this object upon.
   */
  virtual void SetLayer(const ISketchUpLayer *pLayer) THROWS(ISketchUpException&)
    = 0;

  /**
   * \brief Gets the layer upon which the current object lives.
   * \retval The layer for this object.
   */
  virtual ISketchUpLayerPtr GetLayer() const = 0;

  /**
   * \brief Does this element cast shadows?
   * \retval true iff this object casts shadows.
   */
  virtual bool GetCastsShadows() const = 0;

  /**
   * \brief Tell this object whether or not it should cast shadows.
   * \param cast Use 'true' to indicate that this object should cast shadows.
   */
  virtual void SetCastsShadows(bool cast) = 0;
  /**
   * \brief Does this element cast shadows?
   * \retval true iff this object receives shadows.
   */
  virtual bool GetReceivesShadows() const = 0;

  /**
   * \brief Tell this object whether or not it should cast shadows.
   * \param cast Use 'true' to indicate that this object should cast shadows.
   */
  virtual void SetReceivesShadows(bool val) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDRAWINGELEMENT_H_
