// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGE_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGE_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/idrawingelement.h"
namespace atlast {
namespace sketchup {
class ISketchUpVertex;
class ISketchUpFace;

/**
 * @interface ISketchUpEdge
 * @brief An Edge object.
 *
 * Every 2-dimensional object in SketchUp is represented by
 * one or more connected Edge.
 */
class ISketchUpEdge :
  public ISketchUpDrawingElement {
public:
  /**
   * @brief Sets the "smooth" property of an Edge.
   * @param isSmooth Set to true to smooth the edge.
   */
  virtual void SetSmooth(bool isSmooth)  = 0;
  /**
   * @brief Gets the smooth property of an Edge.
   * If this edge is not valid false is returned.
   * @return true iff this Edge is smoothed.
   */
  virtual bool IsSmooth() const         = 0;

  /**
   * @brief Sets the "soft" property of an Edge.
   * @param isSoft Set to true to soften the edge.
   */
  virtual void SetSoft(bool isSoft)      = 0;
  /**
   * @brief Gets the soft property of an Edge.
   *
   * If this edge is not valid false is returned.
   * @return true iff this Edge is softened.
   */
  virtual bool IsSoft() const           = 0;

  /**
   * @brief Retrieves the start vertex of this edge.
   *
   * Returns a NULL pointer if this is not a valid Edge.
   * @return The start vertex of this Edge.
   */
  virtual ISketchUpVertexPtr GetStartVertex() const = 0;
  /**
   * @brief Retrieves the end vertex of this edge.
   *
   * Returns a NULL pointer if this is not a valid Edge.
   * @return The end vertex of this Edge.
   */
  virtual ISketchUpVertexPtr GetEndVertex()   const = 0;

  /**
   * \brief Get the faces that use this edge as a boundary.
   *
   * If no such faces exist, the vector will be empty.
   *
   * If the edge is not valid, the vector will be empty.
   *
   * @return A vector of all the faces using this Edge.
   */
  virtual ISketchUpFaceVectorPtr GetFaces() const = 0;
  /**
   * \brief Get the vertices used by this edge.
   *
   * If the edge is not valid, the vector will be empty.
   *
   * @return A vector of start, then end, vertices of this edge.
   */
  virtual ISketchUpVertexVectorPtr GetVertices() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGE_H_
