// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITY_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITY_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/iobject.h"
#include "source/common/utils/ustring.h"  // AtLast
namespace atlast {
namespace sketchup {

class ISketchUpAttributeDictionary;
class ISketchUpModel;
/**
 * @interface ISketchUpEntity
 * @brief An object belonging to an ISketchUpModel.  Another
 * name for this object might be "ISketchUpModelObject".
 */
class ISketchUpEntity :
  public ISketchUpObject {
public:

  /**
   * @brief Used to retrieve the owning model.
   * @return The model containing (owning) this object.
   */
  virtual ISketchUpModelPtr GetModel() const = 0;

  /**
   * @brief Retrieves the object owning this entity.
   * The owning (or parent) could be the Model or a ComponentDefinition.
   * Either way, this method retrieves the ISketchUpEntityContainer
   * containing the object.
   * @return The owning object.
   */
  virtual ISketchUpEntityContainerPtr GetParent() const = 0;

  /**
   * @brief Attribute creation support.
   *
   * This method will create a attribute dictionary of the given name <b>if</b> there
   * isn't already such a named dictionary. If a dictionary of the same name already
   * exists, this method will return that one.
   *
   * \param name The name of the dictionary to find or create.
   * \return The new or found dictionary.
   */
  virtual ISketchUpAttributeDictionaryPtr CreateAttributeDictionary
    (LPCWSTR name) = 0;

  /**
   * @brief Attribute search support.
   *
   * Searches for an AttributeDictionary of the given name.
   *
   * \param name The name to search on.
   * \return The found dictionary, NULL if not found.
   */
  virtual ISketchUpAttributeDictionaryPtr FindAttributeDictionary
    (LPCWSTR name) const = 0;

  /**
   * @brief Returns the so-called <i>friendly name</i> of the object.
   * @return The type name.
   */
  virtual AtLastUstring GetTypeName() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITY_H_
