// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYFACTORY_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYFACTORY_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include <vector>
#include <utility>
#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/common/geometry/point3d.h"  // AtLast
#include "source/common/geometry/vector3d.h"  // AtLast

#include "source/sketchup/skpwriter/sapi/icurve.h"
#include "source/sketchup/skpwriter/sapi/ivertex.h"
#include "source/sketchup/skpwriter/sapi/iedge.h"
#include "source/sketchup/skpwriter/sapi/iface.h"

namespace atlast {
namespace sketchup {

class ISketchUpEdge;
class ISketchUpComponentInstance;
class ISketchUpComponentDefinition;
class ISketchUpVertex;
class ISketchUpFace;
class ISketchUpPolyline;
class ISketchUpArc;
class ISketchUpCurve;
class ISketchUpLineSegment;


class ISketchUpEntityFactory :
  public ISketchUpEntity {
public:
  virtual ISketchUpEdgePtr CreateEdge(ISketchUpVertex *pStart,
                                      ISketchUpVertex *pEnd)
                                     THROWS(ISketchUpException&)
    = 0;

  virtual ISketchUpVertexPtr CreateVertex(double x, double y, double z) = 0;

  virtual ISketchUpComponentInstancePtr
    CreateComponentInstance(const ISketchUpComponentDefinition
                            *pComponentDefinition)
    THROWS(ISketchUpException&) = 0;

  virtual ISketchUpFacePtr CreateFace(const std::vector<ISketchUpCurvePtr>
                                      &outerLoop)
    THROWS(ISketchUpException&) = 0;

  virtual ISketchUpFacePtr CreateFace(const
                                      std::vector<atlast::geometry::CPoint3d>
                                      &points) = 0;

  virtual ISketchUpFacePtr CreateFace(const std::vector<ISketchUpVertexPtr>
                                      &verts) = 0;

  virtual ISketchUpArcPtr CreateArc(const atlast::geometry::CPoint3d &center,
                                    const atlast::geometry::CPoint3d &startPoint,
                                    const atlast::geometry::CPoint3d &endPoint,
                                    const atlast::geometry::CUnitVector3d
                                    &normal, int numSegs = 12) = 0;

  virtual ISketchUpPolylinePtr CreatePolyline(const std::vector<
                                              ISketchUpLineSegmentPtr>&) = 0;

  virtual ISketchUpLineSegmentPtr CreateLineSegment(ISketchUpVertex *pStart,
                                                    ISketchUpVertex *pEnd) = 0;

  virtual std::vector<ISketchUpVertexPtr> GetVertices() =0;
  virtual std::vector<ISketchUpEdgePtr> GetEdges() =0;
  virtual std::vector<ISketchUpFacePtr> GetFaces() =0;
};
}  // end namespace sketchup
}  // end namespace atlast
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYFACTORY_H_
