// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IGROUP_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IGROUP_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/idrawingelement.h"
#include "source/common/utils/ustring.h"  // AtLast
namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpGroup
 * @brief An Group object.
 *
 */
class ISketchUpGroup : public ISketchUpDrawingElement {
 public:
  virtual ISketchUpEntityContainerPtr GetEntityContainer() const = 0;
  virtual atlast::geometry::CTransformation GetTransformation() const = 0;
  /**
   * \brief Sets the transform of the group relative its definition.
   *
   * This method should be contrasted with ISketchUpController::Transform(), where
   * the group would be transformed from its current location by the
   * given transformation.  In this case, its current location is overwritten, and
   * its location - relative to its definition - is set explicitly.
   *
   * This method is particularly useful when creating new groups, such
   * as in the case of writing an Importer.
   *
   * If this object is not valid, this is a no-op.
   *
   * \param trans The transform from the definition to the desired location of the
   * group.
   */
  virtual void SetTransformation(const atlast::geometry::CTransformation
                                 &trans) = 0;

  virtual AtLastUstring GetName() const = 0;
  virtual void SetName(LPCWSTR name) = 0;

  virtual AtLastUstring GetDescription() const = 0;
  virtual void SetDescription(LPCWSTR desc) = 0;

  virtual ISketchUpEntityVectorPtr Explode() = 0;
  virtual void SetLocked(bool lock) = 0;
  virtual bool IsLocked() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IGROUP_H_
