// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IINPUTPOINT_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IINPUTPOINT_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/iobject.h"
#include "source/common/utils/ustring.h"  // AtLast
namespace atlast { namespace sketchup {
class ISketchUpView;

/**
 * @interface ISketchUpInputPoint
 * @brief Used to gather the current mouse point over the
 * model, complete with inferencing.
 */
class ISketchUpInputPoint : public ISketchUpObject {
 public:
    virtual bool IsValid() const = 0;
    virtual bool HasDisplay() const = 0;
    virtual void Clear() = 0;

    virtual int DegreesOfFreedom() const = 0;

    virtual int GetNestingDepth() const = 0;
    virtual bool Draw(ISketchUpView* pView) = 0;

    virtual ISketchUpEdgePtr GetEdge() const = 0;
    virtual ISketchUpFacePtr GetFace() const = 0;
    virtual ISketchUpVertexPtr GetVertex() const = 0;
    virtual ISketchUpGeometricElementPtr GetGeometricElement() const = 0;

    /** 
     * @brief Used to pick based on the given (x,y) screen coordinates.
     * @param pView The view to use to pick.
     * @param x The x-value in screen coordinates.
     * @param y The y-value in screen coordinates.
     * @param pInputpoint Another input point to use.
     * @return true iff the pick found an entity.
     */
    virtual bool Pick(ISketchUpView* pView,
                          long x, long y,
                          ISketchUpInputPoint* pInputpoint = NULL) = 0;

    virtual atlast::geometry::CPoint3d GetPosition() const = 0;
    virtual atlast::geometry::CTransformation GetTransformation() const = 0;

    virtual AtLastUstring GetToolTip() = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IINPUTPOINT_H_
