// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/common/utils/ustring.h"  // AtLast

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpLayer
 * @brief A layer.
 */
class ISketchUpLayer : public ISketchUpEntity {
 public:
  /**
   * @brief Get the name of this layer.
   *
   * If this layer is not valid, an empty string is returned.
   * @return The name of the layer.
   */
  virtual AtLastUstring GetName() const = 0;

  /**
   * @brief Determine whether this layer is visible.
   * 
   * If this layer is not valid, false is returned.
   *
   * @return true iff this layer is visible.
   */
  virtual bool IsVisible() const = 0;


  /**
   * @brief Set the layer's visibility.
   *
   * @param is_visible If true, the layer will be turned on.
   */
  virtual void SetVisibility(bool is_visible) = 0;

  /**
   * @brief Retrieves the material associated with this layer.
   *
   * It is possible that the returned material will be NULL,
   * in which case the Model's default color should be used.
   *
   * If this layer is not valid, NULL is returned.
   *
   * @return The material used by this layer.
   */
  virtual ISketchUpMaterialPtr GetMaterial() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYER_H_
