// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYERS_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYERS_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpPage
 * @brief A <i>Page</i> is a named view in SketchUp.
 *
 */
class ISketchUpLayers : public ISketchUpEntity {
 public:
  /**
   * \brief The number of Layers in this collection
   */
  virtual int GetCount() const = 0;

  /**
   * \brief The Layer at the given index.
   * \param indx The index of the desired Layer.
   * \return The Layer at the given index or NULL if a bad index was provided.
   */
  virtual ISketchUpLayerPtr GetAt(int indx) const = 0;

  /**
   * \brief Create a new Layer.
   *
   * The given name may be munged to ensure we create a uniquely named Layer.
   */
  virtual ISketchUpLayerPtr CreateLayer(LPCWSTR name = NULL) = 0;

  /**
   * \brief Get a layer by name.
   * \return NULL iff there is no layer with the given name, otherwise, returns the layer.
   */
  virtual ISketchUpLayerPtr FindLayer(LPCWSTR name) = 0;

  /**
   * Delete the given Layer.
   */
  virtual void DeleteLayer(ISketchUpLayer *pLayer) = 0;

  /**
   * Set the visibility of the given layer.
   * If the layer is the current active layer, this method will only succeed if there is another layer to 
   * make active.
   *
   * \param pLayer The layer to affect.
   * \param isVisible Use true to turn the layer on, false to suggest that it be turned off.
   * \return true iff the operation was successful.
   */
  virtual bool SetVisible(ISketchUpLayer *pLayer, bool isVisible) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILAYERS_H_
