// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILENGTHFORMATTER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILENGTHFORMATTER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/common/utils/ustring.h"  // AtLast

namespace atlast {
namespace sketchup {
class ISketchUpLengthFormatter;
class ISketchUpInputPoint;

/**
 * @class ISketchUpLengthFormatter
 * @brief A utility class that helps with formatting lengths, or distances.
 */
class ISketchUpLengthFormatter : public ISketchUpObject {
 public:

  /**
   * @brief Formats the given value into a String representation based
   * on the Document's current units settings.
   *
   * @param length The value to format.  TODO: Can it be negative?
   * @param precision An optional argument that dictates the precision
   * to be used in the formatting.  The value must be > 0 and <= 14.
   * @return The formatted length value.
   */
  virtual AtLastUstring Format(double length, int precision = 14) const = 0;
  /**
   * @brief Parses the given string representation of a length, putting
   * the result in the given double value.
   * 
   * This method will attempt to parse the given string which should represent
   * a length.  The result goes in the given double value if not NULL.
   *
   * @param strValue The string representation to be parsed.
   * @param pVal The optional parameter that will be populated with the 
   * parsed value if not NULL and if the strValue parameter is valid.
   * @return true iff the value could be parsed as a length.
   */
  virtual bool Parse(LPCWSTR strValue, double *pVal) const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ILENGTHFORMATTER_H_
