// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMATERIALS_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMATERIALS_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpPage
 * @brief A <i>Page</i> is a named view in SketchUp.
 *
 */
class ISketchUpMaterials : public ISketchUpEntity {
 public:
  /**
   * \brief The number of materials in this collection
   */
  virtual int GetCount() const = 0;

  /**
   * \brief The Material at the given index.
   * \param indx The index of the desired material.
   * \return The material at the given index or NULL if a bad index was provided.
   */
  virtual ISketchUpMaterialPtr GetAt(int indx) const = 0;

  /**
   * \brief Create a new Material.
   *
   * The given name may be munged to ensure we create a uniquely named material.
   */
  virtual ISketchUpMaterialPtr CreateMaterial(LPCWSTR name = NULL) = 0;

  /**
   * Purge the list of unused materials.
   */
  virtual void Purge() = 0;

  /**
   * Delete the given material.
   */
  virtual void DeleteMaterial(ISketchUpMaterial *pMaterial) = 0;
};
}
}

#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMATERIALS_H_
