// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMODEL_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMODEL_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/common/geometry/point3d.h"  // AtLast
#include "source/sketchup/utils/cameradefinition.h"  // SketchUp
#include "source/sketchup/skpwriter/sapi/iobject.h"

namespace atlast {
namespace sketchup {
class ISketchUpAttributeDictionary;
class ISketchUpLayer;
class ISketchUpComponentDefinition;
class ISketchUpOptionsProvider;
class ISketchUpMaterial;

class ISketchUpModel : public ISketchUpObject {
 public:

  /** Get the layers object */
  virtual ISketchUpLayersPtr GetLayers() const = 0;

  /** Get the pages object */
  virtual ISketchUpPagesPtr GetPages() const = 0;

  /**
   * Get the root container of the model.
   * \return The root container of the model.
   */
  virtual ISketchUpEntityContainerPtr GetRoot() const = 0;

  /**
   * Get the active container of the model.  Usually, it's the root, but 
   * if someone is editing a component it will be that component.
   * \return The active container.
   */
  virtual ISketchUpEntityContainerPtr GetActiveContainer() const = 0;

  /**
   * Get the definitions.
   */
  virtual atlast::sketchup::ISketchUpDefinitionsPtr GetComponentDefinitions
    () const = 0;

  /**
   * \brief Returns the ISketchUpOptionsManager owned by this
   * object.
   *
   * \return The owned ISketchUpOptionsManager.
   */
  virtual ISketchUpOptionsManagerPtr GetOptionsManager() const = 0;

  /**
   * \brief Access to the Materials object.
   *
   * Retrieves the collection of materials used by this model.
   * \return The collection of materials used by this model.
   */
  virtual ISketchUpMaterialsPtr GetMaterials() const = 0;

  /**
   * @brief Attribute support.
   *
   * This method will create a attribute dictionary of the given name <b>if</b> there
   * isn't already such a named dictionary. If a dictionary of the same name already
   * exists, this method will return that one.
   *
   * \param name The name of the dictionary to find or create.
   * \return The new or found dictionary.
   */
  virtual ISketchUpAttributeDictionaryPtr CreateAttributeDictionary
    (LPCWSTR name) = 0;

  /**
   * @brief Attribute search support.
   *
   * Searches for an AttributeDictionary of the given name.
   *
   * \param name The name to search on.
   * \return The found dictionary, NULL if not found.
   */
  virtual ISketchUpAttributeDictionaryPtr FindAttributeDictionary
    (LPCWSTR name) const = 0;

  /**
   * @brief Get the document that controls this model.
   * @return The owning document.
   */
  virtual ISketchUpDocumentPtr GetDocument() const = 0;

  /**
   * Set the camera on the model.
   * \param defn The definition of the camera.
   */
  virtual void SetCamera(const CCameraDefinition &defn) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IMODEL_H_
