// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSMANAGER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSMANAGER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {

/**
* \brief A collection of OptionProviders
*
* For a typical model we might find:
\verbatim
- <Options>
- <OptionsProvider name="PageOptions">
<Option name="ShowTransition" value="true" /> 
<Option name="TransitionTime" value="1.5" /> 
</OptionsProvider>
- <OptionsProvider name="AxisOptions">
<Option name="DisplayAxes" value="true" /> 
</OptionsProvider>
- <OptionsProvider name="SlideshowOptions">
<Option name="LoopSlideshow" value="true" /> 
<Option name="SlideTime" value="1" /> 
</OptionsProvider>
- <OptionsProvider name="UnitsOptions">
<Option name="LengthPrecision" value="4" /> 
<Option name="LengthFormat" value="1" /> 
<Option name="LengthUnit" value="0" /> 
<Option name="LengthSnapEnabled" value="true" /> 
<Option name="LengthSnapLength" value="0.25" /> 
<Option name="AnglePrecision" value="1" /> 
<Option name="AngleSnapEnabled" value="true" /> 
<Option name="SnapAngle" value="15" /> 
<Option name="SuppressUnitsDisplay" value="false" /> 
<Option name="ForceInchDisplay" value="false" /> 
</OptionsProvider>
- <OptionsProvider name="RenderingOptions">
<Option name="BackgroundColor" value="unknown type" /> 
<Option name="BandColor" value="unknown type" /> 
<Option name="ConstructionColor" value="unknown type" /> 
<Option name="DepthQueWidth" value="4" /> 
<Option name="DisplayColorByLayer" value="false" /> 
<Option name="DisplayFog" value="false" /> 
<Option name="DisplayFogEdges" value="false" /> 
<Option name="DisplayInstanceAxes" value="false" /> 
<Option name="DrawDepthQue" value="false" /> 
<Option name="DrawGround" value="false" /> 
<Option name="DrawHidden" value="false" /> 
<Option name="DrawHorizon" value="false" /> 
<Option name="DrawLineEnds" value="false" /> 
<Option name="DrawProfilesOnly" value="false" /> 
<Option name="DrawSilhouettes" value="true" /> 
<Option name="DrawUnderground" value="true" /> 
<Option name="EdgeColorMode" value="1" /> 
<Option name="EdgeDisplayMode" value="1" /> 
<Option name="ExtendLines" value="false" /> 
<Option name="FaceBackColor" value="unknown type" /> 
<Option name="FaceColorMode" value="0" /> 
<Option name="FaceFrontColor" value="unknown type" /> 
<Option name="FogColor" value="unknown type" /> 
<Option name="FogDensity" value="0.0001" /> 
<Option name="FogEnd" value="100" /> 
<Option name="FogStart" value="0" /> 
<Option name="FogType" value="2" /> 
<Option name="FogUseBkColor" value="true" /> 
<Option name="ForegroundColor" value="unknown type" /> 
<Option name="GroundColor" value="unknown type" /> 
<Option name="GroundTransparency" value="50" /> 
<Option name="HideAnnotations" value="false" /> 
<Option name="HideConstructionGeometry" value="false" /> 
<Option name="HighlightColor" value="unknown type" /> 
<Option name="HorizonColor" value="unknown type" /> 
<Option name="InactiveHidden" value="false" /> 
<Option name="InstanceHidden" value="false" /> 
<Option name="JitterEdges" value="false" /> 
<Option name="LineEndWidth" value="5" /> 
<Option name="LineExtension" value="8" /> 
<Option name="LockedColor" value="unknown type" /> 
<Option name="MaterialTransparency" value="false" /> 
<Option name="ModelTransparency" value="false" /> 
<Option name="RenderMode" value="3" /> 
<Option name="SectionActiveColor" value="unknown type" /> 
<Option name="SectionCutWidth" value="4" /> 
<Option name="SectionDefaultCutColor" value="unknown type" /> 
<Option name="SectionInactiveColor" value="unknown type" /> 
<Option name="ShowViewName" value="true" /> 
<Option name="SilhouetteWidth" value="3" /> 
<Option name="SkyColor" value="unknown type" /> 
<Option name="Texture" value="true" /> 
<Option name="TransparencySort" value="0" /> 
</OptionsProvider>
- <OptionsProvider name="ShadowOptions">
<Option name="City" value="Boulder (CO)" /> 
<Option name="Country" value="USA" /> 
<Option name="Dark" value="20" /> 
<Option name="DaylightSavings" value="false" /> 
<Option name="DisplayNorth" value="false" /> 
<Option name="DisplayOnAllFaces" value="true" /> 
<Option name="DisplayOnGroundPlane" value="true" /> 
<Option name="DisplayShadows" value="false" /> 
<Option name="EdgesCastShadows" value="false" /> 
<Option name="Latitude" value="40.017" /> 
<Option name="Light" value="80" /> 
<Option name="Longitude" value="-105.283" /> 
<Option name="NorthAngle" value="0" /> 
<Option name="ShadowTime" value="0 0" /> 
<Option name="SunRise" value="0 2" /> 
<Option name="SunSet" value="0 1" /> 
<Option name="TZOffset" value="-7" /> 
<Option name="UseSunForAllShading" value="false" /> 
</OptionsProvider>
</Options>
\endverbatim
*/
class ISketchUpOptionsManager : public ISketchUpEntity {
 public:
  /**
   * \brief Retrieves the number of ISketchUpOptionsProvider objects
   * managed by this object.
   *
   * \return The number of managed ISketchUpOptionsProvider objects.
   */
  virtual unsigned int GetOptionProviderCount() const = 0;

  /**
   * \brief Returns the i'th ISketchUpOptionProvider
   *
   * \return The i'th ISketchUpOptionsProvider if i >= 0 and
   * i < number of managed providers, NULL otherwise.
   */
  virtual ISketchUpOptionsProviderPtr GetOptionsProvider
    (unsigned int indx) const = 0;

  /**
   * \brief Gets the option provider of the given name.
   *
   * \return The option provider of the given name if it exists, NULL otherwise.
   */
  virtual ISketchUpOptionsProviderPtr GetOptionsProvider
    (LPCWSTR providerName) const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSMANAGER_H_
