// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSPROVIDER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSPROVIDER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/sketchup/skpwriter/sapi/ientity.h"

#include "source/sketchup/utils/typedvalue.h"  // SketchUp
#include "source/common/utils/ustring.h"  // AtLast

namespace atlast {
namespace sketchup {
class ISketchUpOptionsProvider : public ISketchUpEntity {
 public:
  virtual ~ISketchUpOptionsProvider() {}

  /**
   * \brief Set the option of the given name to the given value.
   *
   * If the name doesn't match an existing option name (and this
   * <b>is</b> case sensitive) this method is a no-op.
   *
   * \param name The name of the option.
   * \param value The value for the option.  It must be compatible
   * with the option type, or else this is a no-op.
   */
  virtual void SetOption(const char *name, const atlast::utils::CTypedValue
                         &value) = 0;

  /**
   * \brief Get the option of the given name.  Returns an empty typedvalue if the name does
   * not match an option.
   * \param name The name of the desired option.
   * \retval The value of the named option.
   */
  virtual atlast::utils::CTypedValue GetOption(const char *name) const = 0;

  /**
   * \brief Get the name of this options provider.
   * \return The name of this object, empty if this is not a valid object.
   */
  virtual AtLastUstring GetName() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IOPTIONSPROVIDER_H_
