// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGE_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGE_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/common/utils/ustring.h"  // AtLast

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpPage
 * @brief A <i>Page</i> is a named view in SketchUp.
 *
 */
class ISketchUpPage : public ISketchUpEntity {
 public:
  /** Does this page have a camera? */
  virtual bool UsesCamera() const = 0;

  /** Tell this page whether it should use a camera */
  virtual void UseCamera(bool useCamera) = 0;

  /** Get the current camera */
  virtual CCameraDefinition GetCamera() const = 0;

  /** Set the camera for the page (resets the use camera field to true) */
  virtual void SetCamera(const CCameraDefinition &camera) = 0;

  /** Get the page's name */
  virtual AtLastUstring GetName() const = 0;

  /** Set the page's name */
  virtual void SetName(LPCWSTR name) = 0;

  /** Does this page use rendering options? */
  virtual bool UsesRenderingOptions() const = 0;

  /** Tell this page whether it should use Rendering options. */
  virtual void UseRenderingOptions(bool useRendering) = 0;

  /** Get the rendering options (if any) used by this page. */
  virtual ISketchUpOptionsProviderPtr GetRenderingOptions() const = 0;

  /** Does this page use shadow options? */
  virtual bool UsesShadowOptions() const = 0;

  /** Tell this page whether it should use Shadow options. */
  virtual void UseShadowOptions(bool useShadow) = 0;

  /** Get the shadow options (if any) used by this page. */
  virtual ISketchUpOptionsProviderPtr GetShadowOptions() const = 0;

  /**
   * Does this page use Sketch axes?
   *
   * The Sketch coordinate system is referred to as the Sketch axes.
   *
   */
  virtual bool UsesSketchCS() const = 0;

  /** Tell this page whether it should use the Sketch coordinate system. */
  virtual void UseSketchCS(bool use) = 0;

  /**
   * Does this page use the hidden flag?
   */
  virtual bool UsesHidden() const = 0;

  /** Tell this page whether it should use the hidden flag. */
  virtual void UseHidden(bool use) = 0;

  /**
   * Does this page use the hidden layers?
   */
  virtual bool UsesHiddenLayers() const = 0;

  /** Tell this page whether it should use the hidden layers. */
  virtual void UseHiddenLayers(bool use) = 0;
  /**
   * \brief Add a Layer to the array of hidden Layers.
   * \param pLayer A Layer to add to the array.
   */
  virtual void AddLayer(const ISketchUpLayer *pLayer) = 0;

#if 0
    /**
     * \brief Gets the coordinate system used with this page - if any.
     *
     * If this page does not have its own coordinate system, this method
     * returns NULL.
     *
     * \see ISkpPage::UseSketchCS
     * \retval ppCS The Coordinate system used by this page, if any.
     */
    [propget, helpstring("The coordinate system used by this page.")]
    HRESULT CoordinateSystem([out,retval]ISkpCoordinateSystem** ppCS);

    /**
     * \brief [propget] Returns a Collection of all the ISkpLayer objects that have special settings on this Page.
     * This is normally the Layers that are hidden on this Page.
     * \retval pLayers  The ISkpLayers collection.
     * \return
     * S_OK : if successful <br>
     * S_FALSE: if Layers are not used on this Page <br>
     * E_POINTER : if pLayers == NULL <br>
     */
    [propget, helpstring("Get the Layers for this Page")]
    HRESULT Layers([out,retval] ISkpLayers** pLayers);

    /**
     * \brief [propget] Returns a Collection of ISkpEntities that are hidden on this Page.
     * \retval pEntities  The ISkpEntities collection.
     * \return
     * S_OK : if successful <br>
     * S_FALSE: if get_UseHidden returns false <br>
     * E_POINTER : if pLayers == NULL <br>
     */
    [propget, helpstring("Get the hidden Entities for this Page")]
    HRESULT HiddenEntities([out,retval] ISkpEntities** pEntities);

    /**
     * \brief [propget] Returns bitfiled indicating which values are saved.
     * \return
     * S_OK : if successful <br>
     * E_POINTER : if pVal == NULL <br>
     */
    [propget, helpstring("Get the flags indicating which values are saved")]
    HRESULT Flags([out,retval] unsigned long* pVal);

    /**
     * \brief [propget] Gets whether the sketch axes are displayed.
     * \retval pVal is set to TRUE if the axes are displayed.
     * \return
     * S_OK : if successful <br>
     * E_POINTER : if pVal == NULL <br>
     */
    [propget, helpstring("Get whether the sketch axes are displayed")]
    HRESULT SketchAxesDisplayed([out,retval] BOOL* pVal);
#endif
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGE_H_
