// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVERTEX_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVERTEX_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/igeometricelement.h"

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpVertex
 * @brief An interface encapsulating a vertex. A <i>vertex</i> is
 * a topological element that represents a point in model space.
 * It can be used by faces, edges, and edgeuses.
 * @see ISketchUpFace
 * @see ISketchUpEdge
 * @see ISketchUpEdgeUse
 */
class ISketchUpVertex : public ISketchUpGeometricElement {
 public:
  virtual double GetX() const = 0;
  virtual double GetY() const = 0;
  virtual double GetZ() const = 0;
  virtual atlast::geometry::CPoint3d GetPosition() const = 0;
  virtual void PutPosition(atlast::geometry::CPoint3d point) const = 0;

  virtual ISketchUpFaceVectorPtr GetFaces() const = 0;
  virtual ISketchUpEdgeVectorPtr GetEdges() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVERTEX_H_
