// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVIEW_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVIEW_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/defs.h"
namespace atlast {
namespace sketchup {
class ISketchUpView;
class ISketchUpInputPoint;
class ISketchUpPickHelper;

/**
 * @interface ISketchUpView
 * @brief A View class.
 *
 * A view encapsulates the drawing window of the SketchUp<sup>TM</sup> application.
 */
class ISketchUpView : public ISketchUpObject {
 public:
  /**
   * @brief Creates an input point.
   *
   * This method will create an input point associated with this view.
   * Typically, one would create an input point, then use the ISketchUpInputPoint::Pick()
   * method
   * @return The newly created input point.
   */
  virtual ISketchUpInputPointPtr CreateInputPoint() = 0;

  virtual ISketchUpPickHelperPtr CreatePickHelper() = 0;

  virtual void SetToolTip(LPCWSTR text) = 0;

  virtual void Invalidate() = 0;
  virtual bool IsInferenceLocked() const = 0;
  virtual bool LockInference(ISketchUpInputPoint *ip1 = NULL,
                             ISketchUpInputPoint *ip2 = NULL) = 0;

  virtual atlast::geometry::CRay3d GetPickRay(long screenX, long screenY) = 0;

  virtual void DrawLine(const atlast::geometry::CPoint3d &p1,
                        const atlast::geometry::CPoint3d &p2) = 0;

  virtual void DrawLines(atlast::geometry::CPoint3d points[],
                         int numPoints) = 0;

  virtual void SetLineStipple(unsigned short value) = 0;

  typedef enum {
    OpenSquare      = 1,
    ClosedSquare    = 2,
    Plus            = 3,
    X               = 4,
    Star            = 5,
    OpenTriangle    = 6,
    ClosedTriangle  = 7,
  } PointStyle;

  virtual void DrawPoints(atlast::geometry::CPoint3d points[], int numPoints,
                          LPCWSTR color, unsigned int pointSize = 6,
                          PointStyle pointStyle = Plus) = 0;
  virtual void DrawPoints(atlast::geometry::CPoint3d points[], int numPoints,
                          DWORD color, unsigned int pointSize = 6,
                          PointStyle pointStyle = Plus) = 0;

  virtual void DrawPoint(atlast::geometry::CPoint3d point, DWORD color,
                         unsigned int pointSize = 6, PointStyle pointStyle =
                         Plus) = 0;
  virtual void DrawPoint(atlast::geometry::CPoint3d point, LPCWSTR color,
                         unsigned int pointSize = 6, PointStyle pointStyle =
                         Plus) = 0;

  virtual void SetDrawingColor(LPCWSTR color) = 0;
  virtual void SetDrawingColor(DWORD color) = 0;

  virtual void Zoom(double percent) = 0;
  virtual void ZoomExtents() = 0;
};
}
}

#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IVIEW_H_
